// This program reads the analog voltage on PB1 and prints the prints
// the hexadecimal value of this out on the UART.  The UART pins are PA9,PA10

#include "stm32f030xx.h"
#include "serial.h"
#include "nrf24l01.h"
void delay(int dly)
{
	while( dly--)
	{
		asm(" NOP ");asm(" NOP ");
		asm(" NOP ");asm(" NOP ");
		asm(" NOP ");asm(" NOP ");
		asm(" NOP ");asm(" NOP ");
		asm(" NOP ");asm(" NOP ");
		asm(" NOP ");asm(" NOP ");
	}

}
/*************** 
 * ADC Test routines
 **************/  
void initADC()
{

	// Turn on GPIOB
	RCC_AHBENR |= BIT17;
	// Turn on ADC 
	RCC_APB2ENR |= BIT9;	
	// Select analog mode for PA0
	GPIOA_MODER |= (BIT1+BIT0);  
	// Begin ADCCalibration	
	ADC_CR = BIT31;
	// Wait for calibration complete:  
	while ((ADC_CR & BIT31));
	// Select Channel 0
	ADC_CHSELR |= BIT0;
	// Enable the reference voltage
	ADC_CCR |= BIT22;	
	// Enable the ADC
	ADC_CR |= BIT0;  
}
void haltADC()
{
	
	// Select no channel 
	ADC_CHSELR = 0;
	// Disable the reference voltage
	ADC_CCR &= ~BIT22;	
	// Disable the ADC
	ADC_CR |= BIT1;  	
	// Turn off ADC 
	RCC_APB2ENR &= ~BIT9;	
}
void resumeADC()
{
	// Turn on GPIOA
	RCC_AHBENR |= BIT17;
	// Turn on ADC 
	RCC_APB2ENR |= BIT9;	
	// Select analog mode for PA0
	GPIOA_MODER |= (BIT1+BIT0);  
	// Select Channel 9
	ADC_CHSELR |= BIT0;
	// Enable the reference voltage
	ADC_CCR |= BIT22;	
	// Enable the ADC
	ADC_CR |= BIT0;  
}
int readADC()
{
	// Trigger a conversion
	ADC_CR |=  BIT2;
	// Wait for End of Conversion
	while ( (ADC_CR & BIT2) );
	// return result
	return ADC_DR;
}
void testADC()
{
	eputs("ADC: ");
	printHex(readADC());
	eputs("\r\n");
}
void configPins()
{
	// Power up PORTA
	RCC_AHBENR |= BIT17+BIT22;	
	GPIOA_MODER = BIT0+BIT2+BIT4+BIT6+BIT8+BIT10+BIT12+BIT14+BIT26+BIT18+BIT19+BIT20+BIT21; // make bits 0-7 of port A outputs.	
	GPIOF_MODER = BIT0; // Make PF0 an output
	// set all bits high (LEDS off) to start with
	GPIOA_ODR = 0;
	// Turn off pull up resistors on port A (they consume a surprising amount of current)
	GPIOA_PUPDR = 0;
}	
void printRegisters()
{
	int rx;	
	int regnum;
	for (regnum = 0; regnum < 0x20; regnum++)
	{
		printByte(regnum);
		eputs(" : ");
		rx = NRFReadRegister(regnum);
		printByte(rx);
		eputs("\r\n");
		delay(10000); // must delay here to allow serial comms to catch up
	}	    
}
void printPacket(int len,uint8_t *pkt)
{
	while(len--) 
	{
		printByte(*pkt++);
		eputs(" ");
	}
	eputs("\r\n");
}
uint8_t payload[10];
void low_power_mode()
{					
	// Turn off GPIOB,A and F	
	NRFWriteCE(0); // turn radio off
	RCC_AHBENR &= ~(BIT17+BIT18+BIT22);		
	// Turn off ADC 
	haltADC();		
	RCC_CFGR |= 0xf0; // drop bus speed by a factor of 512
	PWR_CR |= BIT0; // switch voltage regulator to low power mode
	cpu_sleep(); // stop cpu
}
void resume_from_low_power()
{	
	RCC_CFGR &= ~0xf0; // speed up to 8MHz	
	// Turn on GPIOB,A and F
	RCC_AHBENR |= BIT18+BIT17+BIT22;
	NRFWriteCE(1); // turn radio back on	
	//Turn on ADC 
	resumeADC();	
}
void RTCISR(void)
{
	RTC_ISR &= ~BIT8;
	EXTI_PR |= BIT17;
}
void initRTC()
{	
	// Turn on power control circuit clock
	RCC_APB1ENR |= BIT28;
	// Turn on LSI
	RCC_CSR |= BIT0;
	// Wait for LSI to be ready
	while ( (RCC_CSR & BIT1) == 0);
	// Unlock the RTC domain
	PWR_CR |= BIT8; // set DBP in PWR_CR
	RCC_BDCR |=  BIT16; // put RTC power domain into reset
	RCC_BDCR &=  ~BIT16; // take it back out of reset
	
	// Turn the RTC on, use LSI
	RCC_BDCR |= BIT15 + BIT9;
	RCC_BDCR &= ~BIT8;
	// Unlock the RTC
	RTC_WPR = 0xca;
	RTC_WPR = 0x53;	
	// RTC Initialization procedure (see reference manual)
	RTC_ISR |= BIT7; // set INIT bit
	while ((RTC_ISR & BIT6)==0); // wait for init to start	
	ISER |= BIT2;   // enable RTC IRQ in NVIC
	
// Alarm interrupt configuration	
	RTC_CR = 0;
	RTC_ALRMAR = BIT31+BIT23+BIT15+BIT7; // ignore all fields -> alarm every second (approx)
	RTC_ALRMASSR = 0x0f000000;  // match all sub second bits
	RTC_CR |= BIT12+BIT8;		// Enable alarm and alarm interrupt
	RTC_ISR &= ~BIT7; 			// clear INIT bit
	
// RTC alarm is an EXTI interrupt in fact so need to configure this too
	EXTI_IMR |= BIT17;  // RTC is triggered via EXTI17,18 or 19
	EXTI_RTSR |= BIT17;
}
int main()
{
	int i;
	int k;		
	configPins();				
	initADC();		
	NRFinit();	
	NRFFlushTX();
	NRFFlushRX();			
	initRTC();
	enable_interrupts();		
	payload[8]=0xaa; // put some known bytes at the end of the payload
	payload[9]=0x54;
	i = 0;
	GPIOF_ODR = 0;	
	SCR  |= BIT2; // enable deep sleep
	while(1) 
	{				
		i = readADC();
		haltADC();		
		payload[1] = i & 0xff;
		payload[0] = (i >> 8);
		NRFWriteData(10,payload);							
		//GPIOF_ODR ^= 0x1;    // uncomment and attach an LED to check timing				
		low_power_mode();	// This will halt cpu and wait for an IRQ				
		resume_from_low_power();
	}    
	return 0;
}
